//
//  LogRotationTest.cpp
//
//  Created by Setsu on 5/24/2010.
//  Copyright 2010 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/Logger.h>
#include <Poco/PatternFormatter.h>
#include <Poco/FormattingChannel.h>
#include <Poco/ConsoleChannel.h>
#include <Poco/SplitterChannel.h>
#include <Poco/FileChannel.h>
#include <Poco/AutoPtr.h>
#include <Poco/Format.h>
#include <Poco/Util/MapConfiguration.h>
#include <Poco/LocalDateTime.h>
#include <Poco/DateTimeFormatter.h>
#include <Poco/Timezone.h>
#include <Poco/Thread.h>

#include <string>

#include "ScopedLogMessage.h"

//----------------------------------------
//	const
//----------------------------------------
const char* const kKeyDefaultValue[][2] = {	{"path",		"rotated.log"}
										,	{"rotation",	"1024"}
										,	{"archive",		"timestamp"}
										,	{"times",		"local"}
										,	{"compress",	"true"}
										,	{"purgeAge",	"10 days"}
										,	{"purgeCount",	"10"}	};

const std::size_t kNumKeyDefaultValue = sizeof(kKeyDefaultValue)/sizeof(kKeyDefaultValue[0]);

//----------------------------------------
//	PrepareConsoleLoggerWithRotatedLogFile
//----------------------------------------
void PrepareConsoleLoggerWithRotatedLogFile(	const std::string& name
											,	const Poco::Util::MapConfiguration* config
											,	int level=Poco::Message::PRIO_INFORMATION)
{
	Poco::AutoPtr<Poco::SplitterChannel> splitterChannel(new Poco::SplitterChannel);
	Poco::AutoPtr<Poco::Channel> consoleChannel(new Poco::ConsoleChannel);
	Poco::AutoPtr<Poco::FileChannel> rotatedFileChannel(new Poco::FileChannel);

	for(std::size_t i=0; i<kNumKeyDefaultValue; ++i)
	{
		rotatedFileChannel->setProperty(kKeyDefaultValue[i][0]
										, config->getString(kKeyDefaultValue[i][0], kKeyDefaultValue[i][1]));
	}

	splitterChannel->addChannel(consoleChannel);
	splitterChannel->addChannel(rotatedFileChannel);

	Poco::AutoPtr<Poco::Formatter> formatter(new Poco::PatternFormatter("%w %b %e %H:%M:%S.%i %Y: %t"));
	formatter->setProperty("times", "local");
	Poco::AutoPtr<Poco::Channel> formattingChannel(new Poco::FormattingChannel(formatter, splitterChannel));

	Poco::Logger::create(name, formattingChannel, level);
}

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	Poco::AutoPtr<Poco::Util::MapConfiguration> config(new Poco::Util::MapConfiguration);
	config->setString("rotation",	"160");
	config->setString("compress",	"false");

	PrepareConsoleLoggerWithRotatedLogFile(	Poco::Logger::ROOT
										,	config
										,	Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("LogRotationTest ", "start", "end");

	for(int i=0; i<10; ++i)
	{
		msg.Message(Poco::format("  LogRotationTest %d", i));
		Poco::Thread::sleep(100);
	}

	return 0;
}
